<?php
/**
*
* FIND - Atom/RSS import
*
* @package functions_find
* @version 0.2r3
* @copyright (c) 2008, 2009 PoPoutdoor
* @license	http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}


/**
* @package functions_find
*/
function get_rss_content($ids = '')
{
	global $config, $db, $user;

	if (empty($ids))
	{
		trigger_error($user->lang['NO_IDS']);
	}

	$single = preg_replace('/[^,]/', '', $ids);
	$query = (empty($single)) ? "= $ids" : "in($ids)";
	$sql = 'SELECT f.*, u.username, u.user_colour, u.user_lang
		FROM ' . FIND_TABLE . ' f, '  . USERS_TABLE . " u 
		WHERE f.robot_id = u.user_id
			AND f.feed_id $query";
	$result = $db->sql_query($sql);

	// init return messages
	$msg = array('ok' => '', 'skip' => '', 'err' => '');

	// Save current user's language name
	$current_lang = $user->lang_name;

	// fetch news from selected feeds and posted into one topic for the same day (from source timestamp)
	while ($row = $db->sql_fetchrow($result))
	{
		// We use Bot user language from here
		$user->lang_name = $row['user_lang'];
		$user->add_lang('mods/find');

		$feed_name = $row['feed_name'];
		// feed not active, skipped to next
		if (!$row['feed_active'])
		{
			$msg['skip'][] = sprintf($user->lang['NOT_ACTIVE'], $feed_name);
			continue;
		}

		// prepare some vars
		$feed_id = $row['feed_id'];
		$subject_feedname = $row['post_feedname'];
		$item_limit = $row['feed_limit'];
		$post_limit = ($row['feed_contents']) ? $row['post_limit'] : -1;
		$inc_channel = $row['feed_channel'];
		$inc_image = $row['feed_image'];
		$last_update = $row['last_visit'];

		$bot = array();
		$bot['post_forum'] = $row['post_forum'];
		$bot['poster_id'] = $row['robot_id'];
		$bot['username'] = $row['username'];
		$bot['post_colour'] = $row['user_colour'];
		$bot['ip'] = $user->data['session_ip'];

		$parser = new rss_parser();
		if ($parser->parse($row['feed_url'], $row['feed_recode']))
		{
			// Get the channel published/updated date, use current time if no timestamp from source
			$channel_date = $parser->channel_date;
			$channel_update = $parser->channel_update;
			$channel_ts = (!empty($channel_update)) ? $channel_update : ((!empty($channel_date)) ? $channel_date : time());

			// jump to next feed if outdated
			if ($last_update >= $channel_ts)
			{
				$msg['skip'][] = sprintf($user->lang['FEED_NONE'], $feed_name);
				continue;
			}

			// init vars
			$contents = $heading = $subject = '';
			$processed = $skipped = $latest_ts = 0;

			// Sort items by timestamp
			$item_ary = $parser->items;
			foreach ($item_ary as $key => $row) 
			{
				$a[$key]  = (!empty($row['pubDate'])) ? $row['pubDate'] : 0;
			}
			array_multisort($a, SORT_DESC, $item_ary);
			unset($a);

			// Determine number of news items to process
			$i = sizeof($item_ary);
			if ($item_limit && ($item_limit < $i) )	
			{ 
				$i = $item_limit;
			}
			$i--;

			// Loop through the list of items, up to the limit.
			for ($i; $i >= 0; $i--)
			{
				// Check if any updates.
				$item_ts = $item_ary[$i]['pubDate'];
				if ($item_ts)
				{
					$latest_ts = ($item_ts > $latest_ts) ? $item_ts : $latest_ts;

					// skip to next item if outdated
					if ($last_update >= $item_ts) 
					{
						$skipped++;
						continue;
					}
				}

				// Check to see if we have a subject - some atom feeds like blogspot provide an empty title.
				$title = utf8_tidy($item_ary[$i]['title'], $parser->cjk);
				if (empty($title))
				{
					$title = utf8_tidy($item_ary[$i]['description'], $parser->cjk);
				}
				$title = truncate_string($title, 60, 255, false, $user->lang['TURNCATE']);

				// prepare the message text
				$message = bb_message(
				   $item_ary[$i], 
				   $item_ts, 
				   $post_limit,
				   $inc_image,
					$parser->cjk
				);

				$processed++;

				$contents .= sprintf($user->lang['BB_CONTENTS'], $title) . $message;

			} // end process items

			// message for skipped items
			if ($skipped)
			{
				$msg['skip'][] = sprintf($user->lang['FEED_SKIP'], $feed_name, $skipped);
			}
			// should we include the newsfeed image
			$image_url = $parser->image_url;
			if ($inc_image && !empty($image_url))
			{
				$image_link = $parser->image_link;

				$heading .= sprintf($user->lang['BB_URL'], (empty($image_link)) ? $parser->items[0]['link'] : $image_link, "[img]${image_url}[/img]") . "\n";
			}

			// should we use the acp feedname as subject?
			if ($subject_feedname)
			{
				$subject = $feed_name;
			}

			// should we include the channel info for this newsfeed
			if ($inc_channel)
			{
				$channel = utf8_tidy($parser->channel, $parser->cjk);
				$channel_desc = utf8_tidy($parser->channel_desc, $parser->cjk);
				$channel_link = $parser->channel_link;

				// Assign subject
				if (empty($subject))
				{
					if (!empty($channel))
					{
						$subject = truncate_string($channel, 60, 255, false, $user->lang['TURNCATE']);
					}
					elseif (!empty($channel_desc))
					{
						$subject = truncate_string($channel_desc, 60, 255, false, $user->lang['TURNCATE']);
					}
					else
					{
						$subject = $feed_name;
					}
				}

				if (!empty($channel) && empty($heading))
				{
					$heading .= sprintf($user->lang['BB_CHANNEL'], (empty($channel_link)) ? $channel : sprintf($user->lang['BB_URL'], $channel_link, $channel)) . "\n";
				}

				if (!empty($channel_desc) && $channel_desc != $channel)
				{
					$heading .= sprintf($user->lang['BB_DESC'], $channel_desc);
				}
			}

			$heading .= sprintf($user->lang['BB_UPDATED'], $user->lang['LAST_UPDATED'], $user->format_date($channel_ts));

			if ($processed)
			{
				// New topic on new day ;)
				$new_topic = ($user->format_date($latest_ts + $config['board_timezone'] + $config['board_dst'], 'Y-m-d', true) 
					== $user->format_date($last_update + $config['board_timezone'] + $config['board_dst'], 'Y-m-d', true)) ? 'reply' : 'post';

				rss_autopost($bot, $new_topic, $subject, $heading . $contents);

				$msg['ok'][] = sprintf($user->lang['FEED_OK'], $feed_name, $processed);
			}

			// update feed last visit time
			if (!$latest_ts)
			{
				$latest_ts = $channel_ts;
			}
			$sql = 'UPDATE ' . FIND_TABLE . '
				SET last_visit = ' . $latest_ts . "
				WHERE feed_id = $feed_id";
			$update_result = $db->sql_query($sql);
			$db->sql_freeresult($update_result);

		}
		else
		{
			$error = $parser->parser_error;
			if (!empty($error))
			{
				$msg['err'][] = sprintf($user->lang['FEED_ERR'], $feed_name, $error[0], $error[1], $error[2], $error[3], $error[4]);
			}

			$error = $parser->error_msg;
			if (!empty($error))
			{
				$msg['err'][] = "[$feed_name] $error";
			}
		}

		$parser->destroy();
		//unset($parser);
	}
	$db->sql_freeresult($result);

	// Restore current user's language
	$user->lang_name = $current_lang;

	return $msg;
}



/*
* Convert supported html tags to BBCode
*
*	Supported html tags(10) are: strong,b,u,i,ul,ol,li,img,a
*/
function html2bb($html, $show_img = true, $cjk = false)
{
//	$html = html_entity_decode($html, ENT_COMPAT, "UTF-8");

	if ($html == strip_tags($html))
	{
		return utf8_tidy($html, $cjk, true);
	}

	// <strong>...</strong>, <b>...</b>, <u>...</u>, <i>...</i>
	//	to [b]...[/b], [b]...[/b], [u]...[/u], [i]...[/i]
	$html = preg_replace('#<(/?)strong>#i', '[\\1b]', $html);		
	$html = preg_replace('#<(/?[biu])>#i', '[\\1]', $html);		
	// <ul><li>...</li></ul> to [list][*]...[/list]
	// <ol><li>...</li></ol> to [list=n][*]...[/list]
	$html = preg_replace('#<(/?)ul>#is', '[\\1list]', $html);
	$html = preg_replace('#<ol\ +type="(\w+)">#is', '[list=\\1]', $html);
	$html = preg_replace('#</ol>#is', '[/list]', $html);
	$html = str_replace('<li>', '[*]', $html);
	$html = str_replace('</li>', "\n", $html);
	// Put additional tags support below

	// strip html tags but <img><a>
	$html = strip_tags($html, '<img><a>');
	$html = url2bb($html, $show_img);
	// strip html tags left behide
	$html = strip_tags($html);
	$html = utf8_tidy($html, $cjk, true);

	return $html;
}



/**
*	convert html <a> tags to BBcode, with additional <img> tags support
*/
function url2bb($html, $show_img = true)
{
	if ($html == strip_tags($html))
	{
		return $html;
	}

	if (!$show_img)
	{
		$html = strip_tags($html, '<a>');
	}

	if (preg_match_all('#<a[^href=("|\')?].*?((?:(?:f|ht)tps?)[^\'">]*)[^>]*>(.*?)</a>#is', $html, $tag, PREG_PATTERN_ORDER))
	{
		$count = sizeof($tag[0]);
		for ($i = 0; $i < $count; $i++)
		{
			// convert img tags first
			if ($show_img)
			{
				$txt = img2bb($tag[2][$i]);
			}
				
			//	Process url
			$url = fix_url($tag[1][$i]);
			// In case url being stripped off, preserve text inside tag
			if (empty($url))
			{
				$bbcode = $txt;
			}
			else if (empty($txt))
			{
				// [url]link[/url]
				$bbcode = "[url]${url}[/url]";
			}
			else
			{
				// [url=link]text[/url]
				$txt = trim($txt);
				$bbcode = "[url=${url}]${txt}[/url]";
			}

			$html = str_replace($tag[0][$i], $bbcode, $html);
		}
	}

	// process image not inside <a> tags
	if ($show_img)
	{
		$html = img2bb($html);
	}

	return $html;	
}



/**
*	Convert html <img> tags to [img]...[/img] BBCode
*	Support file extenstions are: gif, jp2, jpg, png, xbm
*/
function img2bb($html)
{
	if (strip_tags($html) == $html)
	{
		return $html;
	}

	if (preg_match_all('#<img\s*?src=[\'"]?([^(f|ht)tps?]*[^>]*\.(?:gif|jp[2g]|png|xbm)).*?>#is', $html, $tag, PREG_PATTERN_ORDER))
	{
		$count = sizeof($tag[0]);
		for ($i = 0; $i < $count; $i++)
		{
			$url = fix_url($tag[1][$i]);
			$bbcode = (empty($url)) ? '' : "[img]${url}[/img]";

			$html = str_replace($tag[0][$i], $bbcode, $html);
		}
	}

	return $html;	
}



/**
*	URI filter
*/
function fix_url($url)
{
	// Add filter below
/*
	// remove CNN feed's ad link
	$pos = utf8_strpos($url, 'rss.cnn.com/~a/rss');
	if ($pos)
	{
		$url = '';
	}
*/
	return trim($url);
}



/**
*	Format text with supported html tags to BBCode
*/
function bb_message($item, $item_ts, $post_limit, $inc_image, $cjk = false)
{
	global $user;

	$message = ''; 

	$link = $item['link'];
	$category = utf8_tidy($item['category'], $cjk);
	$pubDate = $item_ts;
	$update = $item['update'];
	$author = utf8_tidy($item['author'], $cjk);

	if (!empty($category))
	{
		$message .= sprintf($user->lang['BB_CAT'], $category);
	}

	// Check to see if a category and publish date data have been provided.
	if (!empty($pubDate))
	{
		$message .= sprintf($user->lang['BB_POSTED'], $user->lang['POSTED'], $user->format_date($pubDate));
	}

	// Insert the author information.
	if (!empty($author))
	{
		$message .= (empty($pubDate)) ? '' : '    ';	// empty or 4 spaces
		$message .= sprintf($user->lang['BB_AUTHOR'], $user->lang['AUTHOR'], $author);
	}
	else
	{
		$message .= "\n";
	}

	if (!empty($update) && ($update != $pubDate))
	{
		$message .= sprintf($user->lang['BB_UPDATED'], $user->lang['LAST_UPDATED'], $user->format_date($update));
	}

	// Now we add the content if provided.
	if (($post_limit != -1) && !empty($item['description']))
	{
		$item_content = html2bb($item['description'], $inc_image, $cjk);
		// Contents filter 
		//$item_content = str_replace('Read full story for latest details.', '', $item_content);	// Strip CNN feed's "read more" line
		$item_content = ($post_limit > 0) ? truncate_string($item_content, $post_limit, 255, false, $user->lang['TURNCATE']) : $item_content;

		$message .= "\n${item_content}\n";
	}

	if (!empty($link))
	{
		$message .= sprintf($user->lang['BB_READ_MORE'], $link);
	}

	return $message;
}



/**
*	Cleanup spacing and newlines, with aditional spacing fix-ups for CJK text
*/
function utf8_tidy($text, $cjk = false, $newline = false)
{
	// Skip space fixing if all single byte chars
	//if ($cjk || (utf8_strlen($text) != strlen($text)))
	if ($cjk)
	{
		$text = cjk_tidy($text);
	}

	$text = ($newline) ? preg_replace("#\n{4,}#", "\n\n\n", $text) : str_replace("\n", '', $text);

	return trim($text);
}



/**
* Fix CJK full-width punct-alpnum spacing
*
* utf8 ncr values for CJK full-width symbols:
*	12288 - 12290, 12298 - 12318
*	65281 - 65312
*	65313 - 65338		excludes english capital letters
*	65339 - 65344
*	65345 - 65370		excludes english letters
*	65371 - 65377
*	65504 - 65510
*/
function cjk_tidy($text)
{
	// Preserve space around [] , for posting with bbcode tags
	$text = preg_replace('#\] +([[:punct:]])#', ']&#32;\\1', $text);
	$text = preg_replace('#([[:punct:]]) +\[#', '\\1&#32;[', $text);
	// encolsed words with spaces
	$text = preg_replace('#([[:alnum:][:punct:]\-\+]+)#', '&#32;\\1&#32;', $text);

	$text = utf8_encode_ncr($text);

	$text = preg_replace('/(?:(&#[0-9]{5};)(\w+)|(\w+)(&#[0-9]{5};))/', '\\1 \\2', $text);
	$text = preg_replace('/\]&#32;(&#[0-9]{5};)/', ']\\1', $text);
	$text = preg_replace('/(&#[0-9]{5};)&#32;\[/', '\\1[', $text);

	// restore space
	$text = str_replace('&#32;', ' ', $text);

	// process spacings
	$val = 12287;
	while ($val <= 12318)
	{
		$val++;
		if ($val > 12290 && $val < 12298)
		{
			continue;
		}

		$text = preg_replace(array("/(&#$val;) +/", "/ +(&#$val;)/"), '\\1', $text);
	}

	$val = 65280;
	while ($val <= 65378)
	{
		$val++;
		if ( ($val > 65312 && $val < 65339) || ($val > 65344 && $val < 65371) )
		{
			// skip Full-width letters and part not in range
			continue;
		}

		$text = preg_replace(array("/(&#$val;) +/", "/ +(&#$val;)/"), '\\1', $text);
	}

	$val = 65503;
	while ($val <= 65510)
	{
		$val++;

		$text = preg_replace(array("/(&#$val;) +/", "/ +(&#$val;)/"), '\\1', $text);
	}

	$text = utf8_decode_ncr($text);

	return $text;
}



/**
* Bot Submit Post
* (This is the customized cut down version of 3.0.6 function submit_post)
*
* $mode: post/reply, 'approve' always "true'. Bot user always 'mark read', always update search index.
* No edit, no poll, no attachment, no quote, no notifications for poster, no globalising
*
*/
function rss_autopost($bot, $mode, $subject, $message)
{
	global $db, $auth, $config, $phpEx, $phpbb_root_path;

	$current_time = time();

	// First of all make sure the subject is having the correct length.
	// To achieve this without cutting off between special chars we convert to an array and then count the elements.
	$subject = truncate_string($subject);

	// variables to hold the parameters for submit_post
	$uid = $bitfield = $options = ''; 

	generate_text_for_storage($subject, $uid, $bitfield, $options, false, false, false);
	generate_text_for_storage($message, $uid, $bitfield, $options, true, true, true);

	// Collect some basic information about which tables and which rows to update/insert
	$sql_data = array();

	// Start the transaction here
	$db->sql_transaction('begin');

	// Collect Information
	$sql_data[POSTS_TABLE]['sql'] = array(
		'forum_id'				=> $bot['post_forum'],
		'poster_id'				=> $bot['poster_id'],
		'icon_id'				=> false,
		'poster_ip'				=> $bot['ip'],
		'post_time'				=> $current_time,
		'post_approved'		=> 1,
		'enable_bbcode'		=> true,
		'enable_smilies'		=> false,
		'enable_magic_url'	=> false,
		'enable_sig'			=> false,
		'post_username'		=> $bot['username'],
		'post_subject'			=> $subject,
		'post_text'				=> $message,
		'post_checksum'		=> md5($message),
		'post_attachment'		=> 0,
		'bbcode_bitfield'		=> $bitfield,
		'bbcode_uid'			=> $uid,
		'post_postcount'		=> ($auth->acl_get('f_postcount', $bot['post_forum'])) ? 1 : 0,
		'post_edit_locked'	=> 1
	);

	// We only handle new topic and reply post here.
	if ($mode == 'post')
	{
		$sql_data[TOPICS_TABLE]['sql'] = array(
			'topic_poster'						=> $bot['poster_id'],
			'topic_time'						=> $current_time,
			'topic_last_view_time'			=> $current_time,
			'forum_id'							=> $bot['post_forum'],
			'icon_id'							=> false,
			'topic_approved'					=> 1,
			'topic_title'						=> $subject,
			'topic_first_poster_name'		=> $bot['username'],
			'topic_first_poster_colour'	=> $bot['post_colour'],
			'topic_type'						=> 0,
			'topic_time_limit'				=> 0,
			'topic_attachment'				=> 0,
		);

		$sql_data[USERS_TABLE]['stat'][] = "user_lastpost_time = $current_time" . (($auth->acl_get('f_postcount', $bot['post_forum'])) ? ', user_posts = user_posts + 1' : '');

		$sql_data[FORUMS_TABLE]['stat'][] = 'forum_posts = forum_posts + 1';
		$sql_data[FORUMS_TABLE]['stat'][] = 'forum_topics_real = forum_topics_real + 1' . ', forum_topics = forum_topics + 1';

		// Submit new topic
		$sql = 'INSERT INTO ' . TOPICS_TABLE . ' ' .
		$db->sql_build_array('INSERT', $sql_data[TOPICS_TABLE]['sql']);
		$db->sql_query($sql);

		$topic_id = $db->sql_nextid();

		$sql_data[POSTS_TABLE]['sql'] = array_merge($sql_data[POSTS_TABLE]['sql'], array(
			'topic_id' => $topic_id)
		);
		unset($sql_data[TOPICS_TABLE]['sql']);
	}
	else
	{
		$sql_data[TOPICS_TABLE]['stat'][] = 'topic_last_view_time = ' . $current_time . ',
			topic_replies_real = topic_replies_real + 1,
			topic_bumped = 0,
			topic_bumper = 0, topic_replies = topic_replies + 1';

		$sql_data[USERS_TABLE]['stat'][] = "user_lastpost_time = $current_time" . (($auth->acl_get('f_postcount', $bot['post_forum'])) ? ', user_posts = user_posts + 1' : '');

		$sql_data[FORUMS_TABLE]['stat'][] = 'forum_posts = forum_posts + 1';

		// get topic id
		$sql = 'SELECT topic_id
			FROM ' . TOPICS_TABLE . "
			WHERE topic_title = '" . $db->sql_escape($subject) . "'
				AND forum_id = " . $bot['post_forum'] . "
			ORDER BY topic_id DESC LIMIT 1";
		$result = $db->sql_query($sql);
		$topic_id = $db->sql_fetchfield('topic_id');
		$db->sql_freeresult($result);

		// Submit new post
		$sql_data[POSTS_TABLE]['sql'] = array_merge($sql_data[POSTS_TABLE]['sql'], array(
			'topic_id' => $topic_id)
		);
	}

	$sql = 'INSERT INTO ' . POSTS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_data[POSTS_TABLE]['sql']);
	$db->sql_query($sql);
	$post_id = $db->sql_nextid();

	if ($mode == 'post')
	{
		$sql_data[TOPICS_TABLE]['sql'] = array(
			'topic_first_post_id'		=> $post_id,
			'topic_last_post_id'			=> $post_id,
			'topic_last_post_time'		=> $current_time,
			'topic_last_poster_id'		=> $bot['poster_id'],
			'topic_last_poster_name'	=> $bot['username'],
			'topic_last_poster_colour'	=> $bot['post_colour'],
			'topic_last_post_subject'	=> $subject,
		);
	}

	unset($sql_data[POSTS_TABLE]['sql']);

	// Update the topics table
	if (isset($sql_data[TOPICS_TABLE]['sql']))
	{
		$sql = 'UPDATE ' . TOPICS_TABLE . '
			SET ' . $db->sql_build_array('UPDATE', $sql_data[TOPICS_TABLE]['sql']) . '
			WHERE topic_id = ' . $topic_id;
		$db->sql_query($sql);
	}

	// Update the posts table
	if (isset($sql_data[POSTS_TABLE]['sql']))
	{
		$sql = 'UPDATE ' . POSTS_TABLE . '
			SET ' . $db->sql_build_array('UPDATE', $sql_data[POSTS_TABLE]['sql']) . '
			WHERE post_id = ' . $post_id;
		$db->sql_query($sql);
	}

	// we need to update the last forum information

	// the last post makes us update the forum table. This can happen if...
	// We make a new topic
	// We reply to a topic
	$sql_data[FORUMS_TABLE]['stat'][] = 'forum_last_post_id = ' . $post_id;
	$sql_data[FORUMS_TABLE]['stat'][] = "forum_last_post_subject = '" . $db->sql_escape($subject) . "'";
	$sql_data[FORUMS_TABLE]['stat'][] = 'forum_last_post_time = ' . $current_time;
	$sql_data[FORUMS_TABLE]['stat'][] = 'forum_last_poster_id = ' . $bot['poster_id'];
	$sql_data[FORUMS_TABLE]['stat'][] = "forum_last_poster_name = '" . $db->sql_escape($bot['username']) . "'";
	$sql_data[FORUMS_TABLE]['stat'][] = "forum_last_poster_colour = '" . $db->sql_escape($bot['post_colour']) . "'";

	// topic sync time!
	// simply, we update if it is a reply or the last post is edited
	// reply requires the whole thing
	if ($mode == 'reply')
	{
		$sql_data[TOPICS_TABLE]['stat'][] = 'topic_last_post_id = ' . (int) $post_id;
		$sql_data[TOPICS_TABLE]['stat'][] = 'topic_last_poster_id = ' . $bot['poster_id'];
		$sql_data[TOPICS_TABLE]['stat'][] = "topic_last_poster_name = '" . $db->sql_escape($bot['username']) . "'";
		$sql_data[TOPICS_TABLE]['stat'][] = "topic_last_poster_colour = '" . $db->sql_escape($bot['post_colour']) . "'";
		$sql_data[TOPICS_TABLE]['stat'][] = "topic_last_post_subject = '" . $db->sql_escape($subject) . "'";
		$sql_data[TOPICS_TABLE]['stat'][] = 'topic_last_post_time = ' . (int) $current_time;
	}
	// Update total post count
	else
	{
		set_config_count('num_topics', 1, true);
	}
	set_config_count('num_posts', 1, true);

	// Update forum stats
	$where_sql = array(POSTS_TABLE => 'post_id = ' . $post_id, TOPICS_TABLE => 'topic_id = ' . $topic_id, FORUMS_TABLE => 'forum_id = ' . $bot['post_forum'], USERS_TABLE => 'user_id = ' . $bot['poster_id']);

	foreach ($sql_data as $table => $update_ary)
	{
		if (isset($update_ary['stat']) && implode('', $update_ary['stat']))
		{
			$sql = "UPDATE $table SET " . implode(', ', $update_ary['stat']) . ' WHERE ' . $where_sql[$table];
			$db->sql_query($sql);
		}
	}

	// Committing the transaction before updating search index
	$db->sql_transaction('commit');

	// Index message contents
	// Select the search method and do some additional checks to ensure it can actually be utilised
	$search_type = basename($config['search_type']);

	if (!file_exists($phpbb_root_path . 'includes/search/' . $search_type . '.' . $phpEx))
	{
		trigger_error('NO_SUCH_SEARCH_MODULE');
	}

	if (!class_exists($search_type))
	{
		include("{$phpbb_root_path}includes/search/$search_type.$phpEx");
	}

	$error = false;
	$search = new $search_type($error);

	if ($error)
	{
		trigger_error($error);
	}

	$search->index($mode, $post_id, $message, $subject, $bot['poster_id'], $bot['post_forum']);

	// Mark this topic as posted to
	markread('post', $bot['post_forum'], $topic_id, $current_time, $bot['poster_id']);

	// Mark this topic as read
	markread('topic', $bot['post_forum'], $topic_id, $current_time, $bot['poster_id']);

	if ($config['load_db_lastread'])
	{
		$sql = 'SELECT mark_time
			FROM ' . FORUMS_TRACK_TABLE . '
			WHERE user_id = ' . $bot['poster_id'] . '
				AND forum_id = ' . ($bot['post_forum']);
		$result = $db->sql_query($sql);
		$f_mark_time = (int) $db->sql_fetchfield('mark_time');
		$db->sql_freeresult($result);
	}
	else if ($config['load_anon_lastread'])
	{
		$f_mark_time = false;
	}

	if ($config['load_db_lastread'] || $config['load_anon_lastread'])
	{
		// Update forum info
		$sql = 'SELECT forum_last_post_time
			FROM ' . FORUMS_TABLE . '
			WHERE forum_id = ' . $bot['post_forum'];
		$result = $db->sql_query($sql);
		$forum_last_post_time = (int) $db->sql_fetchfield('forum_last_post_time');
		$db->sql_freeresult($result);

		update_forum_tracking_info($bot['post_forum'], $forum_last_post_time, $f_mark_time, false);
	}

	return;
}


?>
