<?php
/**
*
* FIND - Atom/RSS import
*
* @package rss_parser
* @version 0.2r3
* @copyright (c) 2008, 2009 PoPoutdoor
* @license	http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* @package rss_parser
*/
class rss_parser 
{
	// private
	var $_xml_parser		= null;

	var $_insideitem		= false;
	var $_insidechannel	= false;
	var $_insideimage		= false;
	var $_tag				= '';
	var $_item				= array();
	var $_image_url		= '';
	var $_image_link		= '';
	var $_channel			= ''; // Channel attributes.
	var $_channel_desc	= '';
	var $_channel_link	= '';
	var $_channel_date	= '';
	var $_channel_update	= '';

	// public
	var $channel			= ''; // Channel attributes.
	var $channel_desc		= '';
	var $channel_link		= '';
	var $channel_date		= '';
	var $channel_update	= '';
	var $image_url			= ''; // Channel image attributes.
	var $image_link		= '';
	var $items				= array(); // Array containing all news items.

	var $parser_error		= array(); // xml parser error message
	var $error_msg			= array(); // Error message is problem encountered.
	var $cjk					= false; // Set true if source encoding or force encoding is CJK, current 

	/**
	* our class constructor
	*/
	function rss_parser()
	{
	}

	// Our class destructor (kind of)
	function destroy()
	{
		// destroy our xml parser.
		if ($this->_xml_parser != null)
		{
			xml_parser_free($this->_xml_parser);
			$this->_xml_parser = null;
		}
	}

	/**
	* Convert date string to epoch format
	*
	* @param $rss_date, the input string
	* @return string, in epoch format. Null if not supported
	*/
	function _date2unix($rss_date)
	{
		global $user;

		$ts = trim($rss_date);

		if (empty($ts))
		{
		 	$ts = '';
		}
		// if date string not all digits (epoch), try to conver here
		elseif (preg_replace('/[0-9]/s', '', $ts))
		{
			// convert atom style timestamp (format: '2006-12-27T22:00:00-04:00')
			$epoch = strtotime(preg_replace('/([0-9]{4}-[0-9]{2}-[0-9]{2})T(.*)([+|-][0-9]{2}):/is', "$1 $2 $3", $ts));
			// previous to PHP 5.1.0, strtotime() error is -1
			$ts = ($epoch === -1 || $epoch === false) ? '' : $epoch;
		}

		return $ts;
	}

	/**
	* URI filter. Remember to update fix_url() in finctions_find too ;)
	*
	* @param $url, the URI string
	* @return string
	*/
	function _fix_url($url)
	{
		$url = trim($url);
		// Strip yahoo redirect links
		$pos = utf8_strpos($url, '*http://');
		if ($pos === true)
		{
			$url = utf8_substr($url, $pos+1);
		}
/*
		// Fix Google's URL redirects (RL too long for posting bbcode)
		$pos = utf8_strpos($url, 'url=');
		if ($pos)
		{
			// Strip the redirect part
			return utf8_substr($url, $pos +4);
		}
*/
		return $url;
	}

	/**
	* Convert CR to LF, strip extra LFs and spaces, multiple newlines to 3
	* Filter unsupported html tags
	*
	* @param $text
	* @param $is_html (booleans), option to retains supported html tags or strip all html tags
	* @return string
	*/
	function _filter($text, $is_html = false)
	{
		$text = str_replace(array("\r", '&nbsp;', '&#32;'), array("\n", ' ', ' '), $text);

		if ($is_html)
		{
			$text = html_entity_decode($text, ENT_COMPAT, "UTF-8");
			// <br />, <p>, </p> to "\n"
			$text = preg_replace(array('#<br ?/?>#i', '#</?p>#i'), "\n", $text);		
			// update function html2bb() from function_find for adding html tag support
			$text = strip_tags($text, '<a><img><strong><b><i><u><li><ul><ol>');
		}
		else
		{
			$text = strip_tags($text);
		}

		$text = preg_replace("# +?\n +?#", "\n", $text);
		$text = preg_replace("#\n{4,}#", "\n\n\n", $text);
		$text = preg_replace('# +#', ' ', $text);

		return trim($text);
	}

	/**
	* parse the xml
	*
	* @parm $url, the feed source's link
	* @parm $recode, overides the source's encoding, default null
	* @return booleans, false if error
	*/
	function parse($url, $recode)
	{
		global $user;

		// Make sure we have not already allocated a parser.
		if ($this->_xml_parser != null)
		{
			return false;
		}
/*
		// try and set some system settings to hopefully get round problems 
		// with security consious hosting providers
		@set_time_limit(120);
		@ini_set("safe_mode", "0");
		@ini_set("allow_url_fopen", "1");
		@set_magic_quotes_runtime(0);
		// Mac support
		//@ini_set("auto_detect_line_endings", "1");
		// This may fix sources which set restrictions depends on the "User-Agent"
		//@ini_set("user_agent", "Mozilla/5.0");
*/
		// get xml contents
		$xml = '';

		//	New file read function
		$url = htmlspecialchars($url);
		$fp = @fopen($url, 'rb');
		if ($fp === false)
		{
			// Retrieve HTTP status code
			list($version, $status_code, $msg) = explode(' ',$http_response_header[0], 3);
			$this->error_msg = sprintf($user->lang['FOPEN_ERR'], $url, $status_code, $msg);

			return false;
		}

		while (!feof($fp))
		{
			$xml .= fread($fp, 8192);
		}
		fflush($fp);
		fclose($fp);

		// Determine encoding from xml
		if (!preg_match('/encoding=["\'](.*?)["\']/', $xml, $match))
		{
			if (empty($recode))
			{
				$this->error_msg = sprintf($user->lang['NO_ENCODINGS'], $url);
				return false;
			}
			// Try to add the XML tag if missing encoding info
			$xml = str_replace('<?xml version="1.0" ?>', '', $xml);
			$xml = '<?xml version="1.0" encoding="' . $recode . '"?>' . $xml;
		}

		// override source encoding
		if (!empty($recode))
		{
			$xml = str_replace($match[1], $recode, $xml);
			$encoding = $recode;
		}
		else
		{
			$encoding = strtolower($match[1]);
		}

		if ($encoding != "utf-8")
		{
			// hack xml encoding info before passing to parser
			$xml = str_replace($encoding, "UTF-8", $xml);
			// Convert source to UTF-8
			$xml = utf8_recode($xml, $encoding);
		}
// Uncomment for debug
/*
		// encoding issue
		//$this->error_msg = utf8_htmlspecialchars(utf8_substr($xml, 0, 99));
		// view source
		//$this->error_msg = utf8_htmlspecialchars($xml);
		return false;
*/
		// filter most control characters, preserves "\r" and "\n"
		$xml = utf8_normalize_nfc($xml);

		// init parser
		$prev_error_reporting = error_reporting();
		error_reporting(E_ERROR);

		// Instantiate an xml parser and setup the reserved events.
		$this->_xml_parser = xml_parser_create("UTF-8");	// This set the input/output encoding
			xml_set_object($this->_xml_parser, $this);
			xml_set_element_handler($this->_xml_parser, 'startElement', 'endElement');
			xml_set_character_data_handler($this->_xml_parser, 'characterData');
			xml_parser_set_option($this->_xml_parser, XML_OPTION_SKIP_WHITE, 1);
			//xml_parser_set_option($this->_xml_parser, XML_OPTION_CASE_FOLDING, 1);
			//xml_parser_set_option($this->_xml_parser, XML_OPTION_TARGET_ENCODING, "UTF-8");	// set output encoding for php5
		$ret = xml_parse($this->_xml_parser, $xml, true);

		error_reporting($prev_error_reporting);

		if (!$ret)
		{
			$this->parser_error = array(
				xml_error_string(xml_get_error_code($this->_xml_parser)), 
				xml_get_current_line_number($this->_xml_parser),
				xml_get_current_column_number($this->_xml_parser) + 1,
				xml_get_current_byte_index($this->_xml_parser),
				$url
			);

			return false;
		}

		// Destroy our xml parser.
		xml_parser_free($this->_xml_parser);
		$this->_xml_parser = null;

		// Set CJK flag
		$this->cjk = (in_array($encoding , array('big5', 'big5-hkscs'))) ? true : false;

		return true;
	}

	/**
	* Normalize RDF tags
	*/
	function _stripNameSpace($string)
	{
		$pos = utf8_strpos($string, ":");
		if ($pos === true)
		{
			$string = utf8_substr($string, $pos + 1);
		}

		return $string;
	}

	/**
	* this event is fired when the xml parser comes across and opening element in the xml
	*/
	function startElement($parser, $tagName) 
	{
		// take a copy of the tag name
		$this->_tag = $this->_stripNameSpace($tagName);

		// if we are on a CHANNEL tag
		if ($tagName =='CHANNEL' || $tagName == 'FEED')
		{
			$this->_insidechannel	= true;
			$this->_insideimage		= false;
			$this->_insideitem		= false;
		}
		// if we are on an IMAGE tag
		elseif ($tagName == 'IMAGE') 
		{
			$this->_insidechannel	= false;
			$this->_insideimage		= true;
			$this->_insideitem		= false;
		}
		// if we are on an ITEM tag
		elseif ($tagName == 'ITEM' || $tagName == 'ENTRY') 
		{
			$this->_insidechannel	= false;
			$this->_insideimage		= false;
			$this->_insideitem		= true;
		}
	}

	/**
	* this event is fired when the xml parser comes across a closing element in the xml
	*/
	function endElement($parser, $tagName) 
	{
		// if it's an item then we want to add the item details to our item details arrays
		if ($tagName == 'ITEM' || $tagName == 'ENTRY') 
		{
			$this->_item['title']			= $this->_filter($this->_item['title']);
			$this->_item['link']				= $this->_fix_url($this->_item['link']);
			$this->_item['description']	= $this->_filter($this->_item['description'], true);
			$this->_item['pubDate']			= $this->_date2unix($this->_item['pubDate']);
			$this->_item['update']			= $this->_date2unix($this->_item['update']);
			$this->_item['guid']				= $this->_fix_url($this->_item['guid']);
			$this->_item['category']		= $this->_filter($this->_item['category']);
			$this->_item['author']			= $this->_filter($this->_item['author']);

			array_push($this->items, $this->_item);

			// reset our item detail variables
			$this->_item = array();
			$this->_insideitem = false;
		}
		// if it's an image
		elseif ($tagName == 'IMAGE')
		{
			$this->image_url	= $this->_fix_url($this->_image_url);
			$this->image_link	= $this->_fix_url($this->_image_link);

			// reset our image detail variables
			$this->_image_url		= '';
			$this->_image_link	= '';
			$this->_insideimage	= false;
		}
		// if it's a channel then we want to add the channel details to our channel details fields
		elseif ($tagName == 'CHANNEL' || $tagName == 'FEED')
		{
			$this->channel				= $this->_filter($this->_channel);
			$this->channel_desc		= $this->_filter($this->_channel_desc);
			$this->channel_link		= $this->_fix_url($this->_channel_link);
			$this->channel_date		= $this->_date2unix($this->_channel_date);
			$this->channel_update	= $this->_date2unix($this->_channel_update);

			// reset our channel detail variables
			$this->_channel			= '';
			$this->_channel_desc		= '';
			$this->_channel_link		= '';
			$this->_channel_date		= '';
			$this->_channel_update	= '';
			$this->_insidechannel	= false;
		}
	}

	/**
	* this event is fired for each character of data which is read from the xml
	* use it to read the bits of information we are interested in
	*/
	function characterData($parser, $data) 
	{
		// if we are inside an item
		if ($this->_insideitem) 
		{
			// depending on what tag we are looking at
			// set the appropriate bit variable - notice that the data is added on to the end of the variable,
			// this is because it's read one bit at a time.
			switch ($this->_tag) 
			{
				case 'TITLE':
					$this->_item['title'] .= $data;
				break;

				case 'CONTENT':
				case 'DESCRIPTION':
					$this->_item['description'] .= $data;
				break;

				case 'LINK':
				case 'ID':
					$this->_item['link'] .= $data;
				break;

				case 'GUID':
					$this->_item['guid'] .= $data;
				break;

				case 'CATEGORY':
					$this->_item['category'] .= $data;
				break;

				case 'DATE':
				case 'PUBDATE':
				case 'PUBLISHED':
					$this->_item['pubDate'] .= $data;
				break;

				case 'UPDATED':
					$this->_item['update'] .= $data;
				break;

				case 'AUTHOR':
					$this->_item['author'] .= $data;
				break;
			}
		}
		// if was inside an image
		elseif ($this->_insideimage)
		{
			switch ($this->_tag) 
			{
				case 'URL':
					$this->_image_url .= $data;
				break;

				case 'LINK':
					$this->_image_link .= $data;
				break;
			}
		}
		// if we are inside a channel
		elseif ($this->_insidechannel)
		{
			switch ($this->_tag) 
			{
				case 'TITLE':
					$this->_channel .= $data;
				break;

				case 'DESCRIPTION':
				case 'SUBTITLE':
					$this->_channel_desc .= $data;
				break;

				case 'LINK':
				case 'ID':
					$this->_channel_link .= $data;
				break;

				case 'DATE':
				case 'PUBDATE':
					$this->_channel_date .= $data;
				break;

				case 'UPDATED':
				case 'LASTBUILDDATE':
					$this->_channel_update .= $data;
				break;
			}
		}
	}
}

?>
